/**
 * @file
 * This file contains the JavaScript code to parse the reports.json and the
 * individual JUnit XML report files to render the reports table.
 * @author Sébastien Tourbier
 * @license Apache-2.0
 */

/* Define mapping between MIP federation URL and Name displayed in the report table.*/
const FEDERATION_URL_NAME_MAPPING = {
  "test_data[https://hbpmip.link/]": "Public",
  "test_data[https://qa.hbpmip.link/]": "QA",
  "test_data[https://stroke.hbpmip.link/]": "Stroke",
};

/**
 * Define a function to read a JUnit XML file and return the content in JSON format.
 * @param {string} url - The URL of the JUnit XML file
 * @returns {Array<{name, status}>} - The content of the JUnit XML file in JSON format
 */
async function parseJUnitXML(
  url = "https://hbpmedical.github.io/pytest-mip/report-03-04-2023-06-59/junit-report.xml"
) {
  const parser = new DOMParser();
  return fetch(url)
    .then((response) => response.text())
    .then((text) => {
      const xml = parser.parseFromString(text, "text/xml");
      const testsuite = xml.getElementsByTagName("testsuite")[0];
      const tests = testsuite.getElementsByTagName("testcase");
      const testsArray = Array.from(tests);
      const testsJSON = testsArray
        .map((test) => {
          const name = test.getAttribute("name");
          if (name.includes("test_data")) {
            const status =
              test.getElementsByTagName("failure").length > 0
                ? "failed"
                : "passed";
            for (const [key, value] of Object.entries(
              FEDERATION_URL_NAME_MAPPING
            )) {
              if (name.includes(key)) {
                return { name: value, status: status };
              }
            }
          }
        })
        .filter((test) => test !== undefined);
      console.log(testsJSON);
      return testsJSON;
    })
    .catch((error) => console.error(error));
}

/**
 * Define a function to create the table header.
 */
function createReportTableHeader() {
  // Get the table head
  const tableHead = document.querySelector("#reportsTable thead");
  // Create the header row
  const headerRow = document.createElement("tr");
  // Append the date to the table head
  const dateHeader = document.createElement("th");
  dateHeader.innerHTML = "Date";
  dateHeader.setAttribute("onclick", "sortTableByDate()");
  dateHeader.setAttribute("rowspan", "2");
  headerRow.appendChild(dateHeader);
  // Append the link to the report to the table head
  const linkHeader = document.createElement("th");
  linkHeader.innerHTML = "Pytest HTML Report";
  linkHeader.setAttribute("rowspan", "2");
  headerRow.appendChild(linkHeader);
  // Append the overall status to the table head
  // const statusHeader = document.createElement("th");
  // statusHeader.innerHTML = "Overall Status";
  // statusHeader.setAttribute("rowspan", "2");
  // headerRow.appendChild(statusHeader);
  // Append the federation name to the table head
  const federationHeaderTitle = document.createElement("th");
  federationHeaderTitle.innerHTML = "Federation Status";
  federationHeaderTitle.setAttribute(
    "colspan",
    Object.keys(FEDERATION_URL_NAME_MAPPING).length
  );
  headerRow.appendChild(federationHeaderTitle);

  // Append the first header row to the table head
  tableHead.appendChild(headerRow);

  // Create the header row
  const secondHeaderRow = document.createElement("tr");

  for (const [key, value] of Object.entries(FEDERATION_URL_NAME_MAPPING)) {
    const federationHeader = document.createElement("th");
    federationHeader.innerHTML = value;
    secondHeaderRow.appendChild(federationHeader);
  }
  // Append the header row to the table head
  tableHead.appendChild(secondHeaderRow);
}

/**
 * Define a function to create a table row for a report.
 * @param {Object} report - The report object retrieved from the reports.json file
 * @returns {void}
 * @async
 */
async function createReportTableRow(report) {
  // Get the table body
  const tableBody = document.querySelector("#reportsTable tbody");
  // Create a new row
  const row = document.createElement("tr");
  // Append the date
  const dateCell = document.createElement("td");
  dateCell.textContent = report.date;
  row.appendChild(dateCell);
  // Append the link to the report
  const linkCell = document.createElement("td");
  const link = document.createElement("a");
  link.textContent = "link";
  link.href = report.link;
  link.target = "_blank";
  linkCell.appendChild(link);
  row.appendChild(linkCell);
  // Append the overall status
  // const statusCell = document.createElement("td");
  // if (report.status === "0") {
  //   statusCell.innerHTML = '<span style="color: green">&#10004;</span>';
  // } else {
  //   statusCell.innerHTML = '<span style="color: red">&#9747;</span>';
  // }
  // row.appendChild(statusCell);
  // Parse the JUnit XML report file to show status of each federation
  if (report.junitxml) {
    console.log("Read JUnitXML report: ", report.junitxml);
    const jUnitXMLReport = await Promise.resolve(
      parseJUnitXML(report.junitxml)
    );
    for (const [key, value] of Object.entries(FEDERATION_URL_NAME_MAPPING)) {
      const federationCell = document.createElement("td");
      const test = jUnitXMLReport.find((test) => test.name === value);
      if (test.status === "passed") {
        federationCell.innerHTML = '<span style="color: green">&#10004;</span>';
      } else {
        federationCell.innerHTML = '<span style="color: red">&#9747;</span>';
      }
      row.appendChild(federationCell);
    }
  } else {
    console.log("No JUnitXML report found for: ", report.link);
    for (const [key, value] of Object.entries(FEDERATION_URL_NAME_MAPPING)) {
      const federationCell = document.createElement("td");
      federationCell.innerHTML = '<span style="color: #999;">n/a</span>';
      row.appendChild(federationCell);
    }
  }
  // Append the row to the table body
  tableBody.appendChild(row);
}

/**
 * Define a function to fetch the JSON data and populate the table
 * @param {string} url - The URL of the main reports.json file listing all the reports
 * @returns {void}
 */
function parseJSONReports(
  url = "https://hbpmedical.github.io/pytest-mip/reports.json"
) {
  fetch(url)
    .then((reports) => reports.json())
    .then((reports) => {
      // Create the table header
      createReportTableHeader();

      // Create and add the table row for each report
      reports.reports.forEach(async (report) => {
        createReportTableRow(report);
      });
    })
    .catch((error) => console.error(error));
}
